//
// BNBOT.CPP
//
// Implementation of a virtual base class for Battle.net bots.
// Enhanced for Higher-Order Bots
//
// by Scott Coleman 3/31/98
// modified by Lord`Bacca`[vL] 12/25/2000
//
//


#include "pch.h"
#pragma hdrstop


#define BUFSIZE         2048


// event IDs
#define EID_SHOWUSER            1001
#define EID_JOIN                1002
#define EID_LEAVE               1003
#define EID_WHISPER             1004
#define EID_TALK                1005
#define EID_BROADCAST           1006
#define EID_CHANNEL             1007
#define EID_USERFLAGS           1009
#define EID_WHISPERSENT         1010
#define EID_CHANNELFULL         1013
#define EID_CHANNELDOESNOTEXIST 1014
#define EID_CHANNELRESTRICTED   1015
#define EID_INFO                1018
#define EID_ERROR               1019
#define EID_EMOTE               1023
#define EID_UNIQUENAME          2010




//============================================================================
BnBot::BnBot() {

  log = 0;
  s = INVALID_SOCKET;
  nServerPort = 6112;
  szLoginName[0] = '\0';
  szUniqueName[0] = '\0';
  szPassword[0] = '\0';
  szServerAddr[0] = '\0';
  szHomeChannel[0] = '\0';
  szCurrentChannel[0] = '\0';
  shutdown=false;
  holdsOps=false;
  voteBanMode=false;
  queueRejoinForOps=false;
  loginSuccess=false;
  nUsers=0;
}


//============================================================================
BnBot::~BnBot() {

  LogClose();
}


//============================================================================
void BnBot::SetLogonInfo(char *szUserName,
                         char *szUserPass,
                         char *szServer,
                         short nPort) {

  strcpy(szLoginName, szUserName);
  strcpy(szUniqueName, szUserName);
  strcpy(szPassword, szUserPass);
  strcpy(szServerAddr, szServer);
  nServerPort = nPort;
}


//============================================================================
void BnBot::SetHomeChannel(char *szChannelName) {

  strcpy(szHomeChannel, szChannelName);
}


//============================================================================
int BnBot::Connect() {
  struct sockaddr_in name;
  struct hostent *hp;

  memset(&name, '\0', sizeof(name));
  name.sin_family = AF_INET;
  name.sin_port = htons(nServerPort);

  // if this is a hostname and not an IP address, resolve it
  char *p = szServerAddr;
  while (*p && (isdigit(*p) || (*p == '.'))) {
    p++;
  }

  // non-digit found - assume hostname
  if (*p) {
    hp = gethostbyname(szServerAddr);
    if (hp == 0)
      return 0;         // can't resolve hostname
    memcpy(&name.sin_addr, hp->h_addr, hp->h_length);
  }
  else {
    name.sin_addr.s_addr = inet_addr(szServerAddr);
  }

  // Initialize the connection socket.
  s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (s == INVALID_SOCKET) {
    return 0;
  }

  // Attempt to connect
  if (connect(s, (struct sockaddr *)&name, sizeof(name))) {
    return 0;  // Connection failed.
  }

  // Connected successfully.
  return 1;
}


//============================================================================
int BnBot::Logon() {

  // Logs on to the battle.net server.

  // send ^C, ^D, username, and password
  // ^C gets the server's attention for logon; ^D turns off ECHO
  Send("%c%c%s\r\n%s\r\n", 0x03, 0x04, szLoginName, szPassword);

  return 1;
}


//============================================================================
void BnBot::Disconnect() {

  if (s == INVALID_SOCKET)
    return;

  loginSuccess=false;
  holdsOps=false;
  closesocket(s);
  s = INVALID_SOCKET;
}


//============================================================================
int BnBot::Reconnect() {

  // Attempts to connect/reconnect the bot.
  // This is different than GreetBot, because this will continue trying to
  // reconnect, while GreetBot gives up after 10 unsuccessful attempts.

  int tries = 0;

  for (;;) {
    ++tries;
    if (!Connect()) {
      Sleep(5000);
      continue;
    }
    if (!Logon()) {
      continue;
    }
    return 1;
  }
  return 0;
}


//============================================================================
int __cdecl BnBot::Send(char *lpszFmt, ...) {

  // Function responsible for sending data to Battle.net
  // This is modified to include simple flood protection.

  char szOutStr[MAXTEXTLENGTH];
  va_list argptr;

  va_start(argptr, lpszFmt);
  vsprintf(szOutStr, lpszFmt, argptr);
  va_end(argptr);

  if (send(s, szOutStr, strlen(szOutStr), 0) < 0)
    return 0;

  // You could uncomment the following line to display outgoing text in
  // the bot window:
  
  // printf("%s",szOutStr);
  Sleep(2000 + strlen(szOutStr) * 10); // Flood protection
  return 1;
}


//============================================================================
int BnBot::ParseEvent(char *pszEvent,
                      int *pnEventId,
                      char *pszSpeaker,
                      u_long *puFlags,
                      char *pszEventText) {

  // You shouldn't need to mess with this function, ever.

  if (!pszEvent || !pnEventId || !pszSpeaker || !puFlags || !pszEventText)
    return 1;

  *pszSpeaker = '\0';
  *pszEventText = '\0';
  *puFlags = 0;

  *pnEventId = atoi(pszEvent);

  // some event messages have no speaker or flag fields
  if ((*pnEventId != EID_INFO) && (*pnEventId != EID_CHANNELFULL) &&
      (*pnEventId != EID_CHANNEL) &&
      (*pnEventId != EID_CHANNELDOESNOTEXIST) &&
      (*pnEventId != EID_CHANNELRESTRICTED) && (*pnEventId != EID_ERROR)) {

    char szJunk[MAXSTRINGLENGTH];
    sscanf(pszEvent, "%d %s %s %x", pnEventId, szJunk, pszSpeaker, puFlags);
  }

  // the event text is enclosed in quotes
  char *p = strchr(pszEvent, '"');
  if (p) {
    strncpy(pszEventText, p+1, MAXTEXTLENGTH);
    pszEventText[MAXTEXTLENGTH-1] = '\0';

    // nix the trailing quote
    p = strrchr(pszEventText, '"');
    if (p)
      *p = '\0';
  }

  return 1;
}


//============================================================================
int BnBot::Dispatch(char *szEventMsg) {

  // A Battle.net event was received, send it to the appropriate function.
  // You shouldn't need to modify this function, ever.

  int nEventId;
  u_long uFlags;
  char szSpeaker[MAXSTRINGLENGTH] = "";
  char szEventText[MAXTEXTLENGTH] = "";

  if (!ParseEvent(szEventMsg,
                  &nEventId,
                  szSpeaker,
                  &uFlags,
                  szEventText)) {
    return 0;
  }

  // dispatch to the appropriate event handler
  switch (nEventId) {
    case EID_SHOWUSER:
      OnShowUser(szSpeaker, uFlags, szEventText);
    break;

    case EID_JOIN:
      OnJoin(szSpeaker, uFlags, szEventText);
    break;

    case EID_USERFLAGS:
      OnUserFlags(szSpeaker, uFlags, szEventText);
    break;

    case EID_LEAVE:
      OnLeave(szSpeaker, uFlags, szEventText);
    break;

    case EID_TALK:
      OnTalk(szSpeaker, uFlags, szEventText);
    break;

    case EID_BROADCAST:
      OnBroadcast(szSpeaker, uFlags, szEventText);
    break;

    case EID_CHANNEL:
      OnChannel(szSpeaker, uFlags, szEventText);
    break;

    case EID_WHISPER:
      OnWhisper(szSpeaker, uFlags, szEventText);
    break;

    case EID_WHISPERSENT:
      OnWhisperSent(szSpeaker, uFlags, szEventText);
    break;

    case EID_EMOTE:
      OnEmote(szSpeaker, uFlags, szEventText);
    break;

    case EID_CHANNELFULL:
      OnChannelFull(szSpeaker, uFlags, szEventText);
    break;

    case EID_CHANNELDOESNOTEXIST:
      OnChannelDoesNotExist(szSpeaker, uFlags, szEventText);
    break;

    case EID_CHANNELRESTRICTED:
      OnChannelRestricted(szSpeaker, uFlags, szEventText);
    break;

    case EID_INFO:
      OnInfo(szSpeaker, uFlags, szEventText);
    break;

    case EID_ERROR:
      OnError(szSpeaker, uFlags, szEventText);
    break;

    case EID_UNIQUENAME:
      OnUniqueName(szSpeaker, uFlags, szEventText);
    break;

    default:
      LogWrite("Unhandled event ID %d", nEventId);
      return 0;
    break;
  }

  return 1;
}


//============================================================================
int BnBot::MsgLoop() {

  // The bot's main message loop after it has established a connection.

  int nBufLen=0;
  int nBufPos=0;
  char stageBuf[BUFSIZE];

  if (s == INVALID_SOCKET)
    return 0;

  for (;;) {


	// Handle timer functions
	DWORD dwCurrentTick = GetTickCount();
	TimerProc(dwCurrentTick);

	// Attempt to receive data from Battle.net
    fd_set fds;
    FD_ZERO(&fds);
    FD_SET(s, &fds);
    struct timeval tv;
    tv.tv_sec = 1;
    tv.tv_usec = 0;

    int n = select(s+1, &fds, 0, 0, &tv);
    if (n) {
      int nNumToRead = BUFSIZE-nBufLen-nBufPos;

      if (nNumToRead == 0) {
        memmove(stageBuf, stageBuf+nBufPos, nBufLen);
        nBufPos = 0;
        nNumToRead = BUFSIZE-nBufLen;
      }

      n = recv(s, stageBuf+nBufPos+nBufLen, nNumToRead, 0);
      if (n <= 0) {
        LogWrite("recv() returned %d, error code %u", n, GetLastError());
        return 0;
      }

      nBufLen += n;

      // dispatch all complete messages in the staging buffer
      while (nBufLen > 0) {
        char *m = stageBuf+nBufPos;
        int nMsgLen=0;
        while (nMsgLen < nBufLen) {
          if (m[nMsgLen] == '\n')
            break;
          nMsgLen++;
        }

        nMsgLen++;
        if (nMsgLen > nBufLen)
          break;

        m[nMsgLen-1] = '\0';

		// You can uncomment the following line to display all raw data the bot
		// receives:

		// printf("%s\n",m);
        if (isdigit(*m))
          Dispatch(m);
		else {
			if (strstr(m,"Login incorrect")) {
				printf("ERROR: Login incorrect\n");
				shutdown=true;
				Disconnect();
			}
		}

        nBufLen -= nMsgLen;
        nBufPos += nMsgLen;
      }

      if (!nBufLen)
        nBufPos = 0;
    }
    IdleHook();
  }
  return 1;
}


//=============================================================================
void BnBot::TimerProc(DWORD currentTick) {
	
	// Executes functions which are activated on a timer.
	//
	// They should be sorted in order of priority, for these timers might send
	// information out to Battle.net, and therefore subject to waiting for flood
	// protection mechanisms to work.

	if (!loginSuccess)  // Don't run these functions if the bot hasn't
		return;			// connected successfully.

	// Rejoin for Ops Timer
	if (queueRejoinForOps && currentTick >= rejoinForOpsExpire)
		RejoinForOpsProc();

	// Anti-idles here - Every minute
	if ((currentTick - lastIdle) / 1000 > 60)
		IdleProc();

	// Voteban Timer
	if (voteBanMode && currentTick >= voteBanExpire)
		VotebanProc();

}
//=============================================================================
void BnBot::VotebanProc() {

	// Calculates the results of a voteban, and splatterkicks the user
	// if the majority wants him/her gone.

	voteBanMode=false;		// Disable the voteban mode.

	if (!holdsOps)			// If the bot no longer holds ops, don't do anything.
		return;

	int percent;

	if (nVotesTotal > 0)	// Calculate the percentage. Protects against
	   percent = (nVotes * 100) / nVotesTotal;	// dividing by 0.
	else
	   percent = 0;

	if (percent > 50) {		// If the majority votes "yea", ban.
		Send("Banning %s by popular request...\r\n",szVoteBanUser);
		RandomBanMessage(szVoteBanUser);
	}
	else					// Otherwise allow him/her to stay.
		Send("OK, %s can stay.\r\n",szVoteBanUser);
}
//=============================================================================
void BnBot::IdleProc() {

	// BNX Anti-idles. Complex, huh? I didn't think so.

	Send("/me thinks anti-idle messages are annoying - don't you?\r\n");
	lastIdle=GetTickCount();
}

//=============================================================================
int BnBot::LogOpen(char *lpszFileName) {

  if (log) {
    return 0;
  }

  log = fopen(lpszFileName, "ab");
  if (!log)
    return 0;

  return 1;
}


//=============================================================================
int __cdecl BnBot::LogWrite(char *lpszFmt, ...) {
  va_list argptr;
  char szOutStr[1024];

  if (!log)
    return 0;

  va_start(argptr, lpszFmt);
  vsprintf(szOutStr, lpszFmt, argptr);
  va_end(argptr);

  fprintf(log, "%s\r\n", szOutStr);
  return 1;
}


//=============================================================================
void BnBot::LogClose() {

  if (log)
    fclose(log);
  log = 0;
}


/*****************************************************************************
*
* Virtual Event Handlers
*
******/


//============================================================================
inline int BnBot::OnShowUser(char *szSpeaker, u_long uFlags, char *szEventText) {

  nUsers++;
  return 1;
}

//============================================================================
inline int BnBot::OnJoin(char *szSpeaker, u_long uFlags, char *szEventText) {

  nUsers++;
  return 1;
}

//============================================================================
inline int BnBot::OnUserFlags(char *szSpeaker,
                              u_long uFlags,
                              char *szEventText) {

  if (stricmp(szSpeaker,szUniqueName) == 0)
	  if (uFlags % 16 == 2)
		  holdsOps=true;

  return 1;
}

//============================================================================
inline int BnBot::OnLeave(char *szSpeaker, u_long uFlags, char *szEventText) {

  nUsers--;
  return 1;
}

//============================================================================
inline int BnBot::OnTalk(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnBroadcast(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnChannel(char *szSpeaker, u_long uFlags, char *szEventText) {

  strcpy(szCurrentChannel, szEventText);
  if (stricmp(szEventText,szHomeChannel) == 0)
	  SetLoginSuccess(true);

  nUsers=0;
  holdsOps=false;
  voteBanMode=false;
  return 1;
}

//============================================================================
inline int BnBot::OnWhisper(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnWhisperSent(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnEmote(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnChannelFull(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnChannelDoesNotExist(char *szSpeaker, u_long uFlags,
                                 char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnChannelRestricted(char *szSpeaker, u_long uFlags,
                               char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnInfo(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}

//============================================================================
inline int BnBot::OnError(char *szSpeaker, u_long uFlags, char *szEventText) {

  return 1;
}


//============================================================================
inline int BnBot::OnUniqueName(char *szSpeaker, u_long uFlags, char *szEventText) {

  strcpy(szUniqueName, szSpeaker);
  return 1;
}
//============================================================================
void BnBot::RandomBanMessage(char *szName) {

	// Random splatterkick messages. Now this is interesting...

	srand(GetTickCount());
	int randnum = rand() % 10;

	switch (randnum) {
	case 0:
		Send("/me smells something bad...\r\n");
		Send("/me looks at %s...\r\n",szName);
		Send("/ban %s Ah! Smell's gone!!\r\n",szName);
		break;

	case 1:
	    Send("/me gags %s, stuffs him into a cow suit, then tosses him into a corral with a horny bull.\r\n",szName);
	    Send("/ban %s Moooo!!!!!!!\r\n",szName);
	    break;

	case 2:
		Send("It's April, the season of growing, and there's F-ING weeds are popping up everywhere.\r\n");
		Send("/me spots a weed in %s\r\n",szCurrentChannel);
		Send("/me grabs a bottle of Round-Up and spritzes %s liberally.\r\n",szName);
		Send("/ban %s FSSST! Weed's gone!\r\n",szName);
		break;

	case 3:
		Send("/me says \"YER OUTTA HERE, PAL!\"\r\n");
		Send("/me takes %s by the balls and throws him into The Void.\r\n",szName);
		Send("/ban %s AND STAY OUT!!\r\n",szName);
		break;

	case 4:
		Send("/me intercepts the ball...\r\n");
		Send("/me dribbles...\r\n");
		Send("/me shoots...\r\n");
		Send("/ban %s\r\n",szName);
		Send("/me SCORES! NUTHIN' BUT NET!!\r\n");
		break;

	case 5:
		Send("/me bashes %s's head in with a baseball bat. *BOK*!!\r\n",szName);
		Send("/ban %s\r\n",szName);
		Send("/me wipes the blood off on %s's hair.\r\n",szName);
		break;

	case 6:
		Send("/me casts a Fireball that goes streaking across the channel at %s.\r\n",szName);
		Send("/me watches as %s's corporeal form is enveloped in flame.\r\n",szName);
		Send("/ban %s poof!!\r\n",szName);
		break;

	case 7:
		Send("%s: What is your real name?\r\n",szName);
		Send("%s: What is your quest?\r\n",szName);
		Send("%s: What is the average velocity of a coconut-laden swallow?\r\n",szName);
		Send("/ban %s\r\n",szName);
		Send("I guess he didn't know!\r\n");
		break;
	
	case 8:
		Send("Congratulations, %s - you're the lucky winner of a one-way trip to The Void!\r\n",szName);
		Send("/ban %s don't forget to write!\r\n",szName);
		break;

	case 9:
		Send("/me pulls down the switch on the electric chair.\r\n");
		Send("/ban %s\r\n",szName);
		Send("/me makes an omelette out of %s's brains.\r\n",szName);
		break;
	}
}

void BnBot::RejoinForOpsProc() {
	if (!holdsOps)
		send(s,"/join The Void\r\n",16,0);

	queueRejoinForOps=false;
}