//
// BNBOT.H
//
// Virtual base class for Battle.net bots.
// Modified slightly to work with enhanced capabilities for
// higher-order bots.
//
// by Scott Coleman 3/31/98
// modified by Lord`Bacca`[vL] 12/25/2000
//
// Please see the README.TXT file for information about this
// file.

#ifndef _BNBOT_H_
#define _BNBOT_H_

#define MAXSTRINGLENGTH         512
#define MAXTEXTLENGTH           1024


// Define the BnBot object. This is a basic bot which is responsible
// for connecting and parsing Battle.net events.
//
// This has been modified to keep track of the following information
// in addition to the GreetBot's original BnBot class:
//
//     a) Attribute which makes the bot aware it's holding ops
//     b) An attribute which keeps track if the bot has logged
//        in its home channel successfully.
//     c) The number of users in the channel.
//     d) Timer functions.
//        1) Anti-idle messages
//        2) Votebanning procedure.
//     e) Rudimentary flood protection.
//     f) Attribute which identifies the bot as "shut down".

class BnBot {
private:
  FILE *log;

protected:
  int s;
  char szLoginName[MAXSTRINGLENGTH];
  char szUniqueName[MAXSTRINGLENGTH];
  char szPassword[MAXSTRINGLENGTH];
  char szServerAddr[MAXSTRINGLENGTH];
  char szHomeChannel[MAXSTRINGLENGTH];
  char szCurrentChannel[MAXSTRINGLENGTH];
  char szConfigFile[MAXSTRINGLENGTH];
  char szShitFile[MAXSTRINGLENGTH];
  char szUserFile[MAXSTRINGLENGTH];
  DWORD lastIdle;
  short nServerPort;
  bool holdsOps;
  bool loginSuccess;
  bool queueRejoinForOps;
  DWORD rejoinForOpsExpire;
  char szVoteBanUser[MAXTEXTLENGTH];
  DWORD voteBanExpire;
  bool voteBanMode;
  int nUsers;
  int nVotes;
  int nVotesTotal;

public:
  BnBot();
  ~BnBot();

  int LogOpen(char *lpszFileName);
  int __cdecl LogWrite(char *lpszFmt, ...);
  void LogClose();
  void SetLogonInfo(char *szUserName,
                    char *szUserPass,
                    char *szServer,
                    short nPort=6112);
  void SetHomeChannel(char *szNewChannel);
  void SetConfigFile(char *szNewConfigFile) { strcpy(szConfigFile,szNewConfigFile); }
  void SetUserFile(char *szNewUserFile) { strcpy(szUserFile,szNewUserFile); }
  void SetShitFile(char *szNewShitFile) { strcpy(szShitFile,szNewShitFile); }
  void SetLoginSuccess(bool state) { loginSuccess = state; if (state) lastIdle=GetTickCount(); }
  int Reconnect();
  int Connect();
  int Logon();
  void Disconnect();
  int __cdecl Send(char *szFmt, ...);
  int ParseEvent(char *pszEvent, int *pnEventId, char *pszSpeaker,
                 u_long *puFlags, char *pszEventText);
  int Dispatch(char *szMsg);
  int MsgLoop();
  void IdleProc();
  void TimerProc(DWORD currentTick);
  void VotebanProc();
  void RandomBanMessage(char *szName);
  void RejoinForOpsProc();

  virtual void IdleHook() { };

  virtual int OnShowUser(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnJoin(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnUserFlags(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnLeave(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnTalk(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnBroadcast(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnChannel(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnWhisper(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnWhisperSent(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnEmote(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnChannelFull(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnChannelDoesNotExist(char *szSpeaker, u_long uFlags,
                                    char *szEventText);
  virtual int OnChannelRestricted(char *szSpeaker, u_long uFlags,
                                  char *szEventText);
  virtual int OnInfo(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnError(char *szSpeaker, u_long uFlags, char *szEventText);
  virtual int OnUniqueName(char *szSpeaker, u_long uFlags, char *szEventText);
  bool shutdown;
};

#endif
